#include <stdio.h>
#include "format.h"
#include <math.h>
#include <stdlib.h>

// la structure de donnee d'un point
typedef struct coord 
{
  float x;
  float y;
} point;

void afficher(point** vect, int dim)
{
  int i,j;
  for(i=0;i<dim;i++)
    {
      for(j=0;j<dim;j++)
	printf("(%d,%d)\t",(int)vect[i][j].x,(int)vect[i][j].y);
    }
  printf("\n");
}


// fonctions analytiques simples :
// la distance
float distance(point a, point b) 
{
  return sqrt((a.x-b.x)*(a.x-b.x)+(a.y-b.y)*(a.y-b.y));
}

// le barycentre de deux points ponderes
point bary(point a, int wa, point b, int wb) 
{
  point m;
  int s=wa+wb;
  m.x=(wa*a.x+wb*b.x)/s;
  m.y=(wa*a.y+wb*b.y)/s;
  return m;
}


point* milieux(point* vP, int P) 
{
  int i,N=2*P-1,dim;
  point* vN;
  
  vN=(point*)malloc(sizeof(point)*N);

  dim=P;
  while(dim>0)
    {
      vN[P-dim]=vP[0];
      vN[N-P+dim-1]=vP[dim];
      dim--;
      for(i=0;i<dim;i++)
	vP[i]=bary(vP[i],1,vP[i+1],1);
    };
  free(vP);
  return vN;
}


point** transpose(point** tab, int N) 
{
  point** tab2;
  int i,j;
  
  tab2=(point**)malloc(sizeof(point*)*N*N);
  for(i=0;i<N;i++)
    for(j=0;j<N;j++)
      tab2[i][j]=tab[j][i];
  free(tab);
  return tab2;
}

point **Casteljau(point **ptctrl, int P) 
{
 
  point **HG, **HD, **BG, **BD;
  point **tab, *col;
  int i,j,N=2*P-1;

  HG=(point**)malloc(sizeof(point)*N*N);
  HD=(point**)malloc(sizeof(point)*N*N);
  BG=(point**)malloc(sizeof(point)*N*N);
  BD=(point**)malloc(sizeof(point)*N*N);
  
  
  tab=(point**)malloc(sizeof(point*)*N*N);
  col=(point*)malloc(sizeof(point*)*N);

  for(i=0;i<P;i++)
    for(j=0;j<P;j++)
      tab[2*i][2*j]=ptctrl[i][j];

  for(i=0;i<P;i++) 
    {
      tab[2*i]=milieux(ptctrl[i],P);
    }
  afficher(tab,N);
  tab=transpose(tab,N);
  for(i=0;i<N;i++) 
    {
      for(j=0;j<P;j++)
	col[j]=tab[i][2*j];
      tab[i]=milieux(col,P);
    }  
  tab=transpose(tab,N);

  /* Decoupe en quatre */
  for(i=0;i<P;i++)
    for(j=0;j<P;j++)
      {
	if (i<P && j<P)
	  HG[i][j]=tab[i][j];
	if (i>=P && j>=P)
	  BD[i][j]=tab[i][j];
	if (i<P && j>=P)
	  HD[i][j]=tab[i][j];
	if (i>=P && j<P)
	  BG[i][j]=tab[i][j];
      }

  if (distance(HG[0][0],HG[1][1])>1)
    HG=Casteljau(HG,P);
  if (distance(HD[0][0],HD[1][1])>1)
    HD=Casteljau(HD,P);
  if (distance(BD[0][0],BD[1][1])>1)
    BD=Casteljau(BD,P);
  if (distance(BG[0][0],BG[1][1])>1)
    BG=Casteljau(BG,P);
  
  /* Collage des quatres morceaux
  for(i=0;i<P;i++) 
    for(j=0;j<P;j++) 
      { 
	if (i<P && j<P) 
	  tab[i][j]=HG[i][j]; 
	if (i>=P && j>=P) 
	  tab[i][j]=BD[i][j]; 
	if (i<P && j>=P) 
	  tab[i][j]=HD[i][j]; 
	if (i>=P && j<P) 
	  tab[i][j]=BG[i][j];
      }
      */
  free(HG);
  free(HD);
  free(BD);
  free(BG);

  return(tab);
}
  

/* int** affecte_controle(point** ptctrl, int ptdim, struct image *im) 
{
  
}*/


point **lire_fichier(int *T, char* nom) 
{
  FILE *f;
  point **ptctrl;
  int max;
  int i,j;

  if ((f=fopen(nom,"r"))!=NULL)
    {
      fscanf(f,"%d\n",&(*T));
      max = *T;
      ptctrl=(point**) malloc (sizeof(point)*max*max);
      printf("Yop c'est cool %d \n\n",max);
      for(i=0;i<max;i++) 
	{
	  for(j=0;j<max;j++) 
	    {
	      fscanf(f,"%f %f\n",&ptctrl[i][j].x,&ptctrl[i][j].y);
	    }
	}
      printf("Yop c'est cool \n\n");
      fclose(f);
      return ptctrl;
    }
  return NULL;
}



int main(int argc, char** argv) 
{
  struct image *im;
  int ptdim;
  point** ptctrl;
  /* point** tab; */

  lire_image(argv[0], im);
  ptctrl=lire_fichier(&ptdim,"controle1.txt");
  printf("Yop c'est cool \n\n");
/*  afficher(ptctrl,ptdim);
  tab = Casteljau(ptctrl,ptdim);*/
return 0;
}
  
  
