#include <stdio.h>
#include <stdlib.h>
#include "format.h"
#include <math.h>

#define allouer2D(M,N)   M=(point**)malloc(sizeof(point*)*N);for(i=0;i<N;i++) M[i]=(point*)malloc(sizeof(point)*N);
#define allouer1D(V,N)   V=(point*)malloc(sizeof(point)*N);
#define max(a,b)         (a>b?a:b)
#define ECHELLE 2

// la structure de donnee d'un point
typedef struct coord {
  float x;
  float y;
  int gris;
} point;

// fonctions analytiques simples :
// la distance
float distance(point a, point b) {
  return sqrt((a.x-b.x)*(a.x-b.x)+(a.y-b.y)*(a.y-b.y));
}

// le barycentre de deux points ponderes
point bary(point a, float wa, point b, float wb) {
  point m;
  float s=wa+wb;
  m.x=(wa * a.x + wb * b.x)/s;
  m.y=(wa * a.y + wb * b.y)/s;
  return m;
}

void courbe1D(point* ctrl, int dim, point* courbe, int ech) {
  int i,j,k;
  float t;
  point* temp;

  allouer1D(temp,dim);
    
  for(i=0; i<ech; i++) {
    t=(float)i/(ech-1);
    for(j=0;j<dim;j++)
      temp[j]=ctrl[j];
    j=dim;
    while(j>1) {
      for(k=0;k<j-1;k++)
	temp[k]=bary(temp[k],1-t,temp[k+1],t);
      j--;
    }
    courbe[i]=temp[0];
  }
  
  free(temp);
}

void courbe2D(point** ctrl, int dim, point** Bez, int ech) {
  point** R;
  point *v1, *v2;
  int i,j;

  R=(point**)malloc(sizeof(point*)*dim);
  for(i=0;i<dim;i++)
    R[i]=(point*)malloc(sizeof(point)*ech);
  allouer1D(v1,dim);
  allouer1D(v2,ech);

  for(i=0;i<dim;i++)
    courbe1D(ctrl[i],dim,R[i],ech);
  
  for(i=0;i<ech;i++) {
    for(j=0;j<dim;j++)
      v1[j]=R[j][i];
    courbe1D(v1,dim,v2,ech);
    for(j=0;j<ech;j++)
      Bez[j][i]=v2[j];
  }
  
  free(v1);
  free(v2);
  for(i=0;i<dim;i++)
    free(R[i]);
  free(R);
}


void colorie(struct image *Im, point** Bez, int ech) {
  int i,j;
  int x,y;
  int dx,dy;

  dx=Im->nb_col;
  dy=Im->nb_lig;

  for(i=0;i<ech-1;i++)
    for(j=0;j<ech;j++) 
      {
	x=(int)dx*Bez[i][j].x;
	y=(int)dy*Bez[i][j].y;
	Bez[i][j].gris=Im->matrice.pgm[x][y];
      }
}

void creer_image(struct image *Im, point **Bez, int ech) {
  int i,j;
  int x,y;
  int dx,dy;

  dx=Im->nb_col;
  dy=Im->nb_lig;
 /* Initialisation */
  for(i=0;i<dx;i++) 
    for(j=0;j<dy;j++) 
      Im->matrice.pgm[i][j]=0;

  for(i=0;i<ech-1;i++)
    {
      for(j=0;j<ech;j++) 
      {
	x=(int)dx*Bez[i][j].x;
	y=(int)dy*Bez[i][j].y;
	Im->matrice.pgm[x][y]=Bez[i][j].gris;
      }
    }
}


void sauv_bez(struct image *Im,point **Bez, int ech,int dx, int dy) 
{
  int i,j,x,y;

  for(i=0;i<dx;i++) 
    for(j=0;j<dy;j++) 
      Im->matrice.pgm[i][j]=0;
  
  for(i=0;i<ech-1;i++)
      for(j=0;j<ech;j++) 
      {
	x=(int)dx*Bez[i][j].x;
	y=(int)dy*Bez[i][j].y;
	Im->matrice.pgm[x][y]=255;
      }
  ecrire_image("Courbe.pgm",Im);
}


int main(int argc, char** argv){
  struct image *Im;
  int dim, i, j, ech;
  int echelle=1;
  point **ctrl;
  point **ctrl2;
  point **Bez1;
  point **Bez2;
  FILE *f;

  if (argc!=4)
    {
      printf("Usage :\ntp image.pgm toto.pgm echelle\n");
      return(-1);
    }
  echelle = atoi(argv[3]);
  Im=(struct image*)malloc(sizeof(struct image));

  if(lire_image(argv[1], Im))
    printf("Erreur !\n");

  printf("%d %d\n", Im->nb_col, Im->nb_lig);
  printf("Chargement des premiers points de controle\n");
  f=fopen("controle1.txt","r");
  fscanf(f,"%d",&dim);
  
  allouer2D(ctrl, dim);

  for(i=0;i<dim;i++) 
    {
    for(j=0;j<dim;j++) 
      {
	fscanf(f,"%f%f",&ctrl[i][j].x,&ctrl[i][j].y);
	printf("%f %f\n",ctrl[i][j].x,ctrl[i][j].y);
      }
    }
  fclose(f);
  
  ech=echelle*max(Im->nb_col,Im->nb_lig);
  allouer2D(Bez1,ech);

  // construction de la surface de Bezier "Bez1"
  printf("Construction de la surface de Bezier 1 \n");
  courbe2D(ctrl, dim, Bez1, ech);
  
  // plaquage Bezier/image, ie affectation des champs "gris" dans "surface"
  printf("Plaquage Bezier/image \n");
  colorie(Im, Bez1, ech);

  // lecture de la deuxieme grille de controle
  printf("Chargement des deuxiemes points de controle\n");
  f=fopen("controle2.txt","r");
  fscanf(f,"%d",&dim);
  
  allouer2D(ctrl2, dim);

  for(i=0;i<dim;i++) 
    {
    for(j=0;j<dim;j++) 
      {
	fscanf(f,"%f%f",&ctrl2[i][j].x,&ctrl2[i][j].y);
	printf("%f %f\n",ctrl2[i][j].x,ctrl2[i][j].y);
      }
    }
  fclose(f);

  ech=echelle*max(Im->nb_col,Im->nb_lig);
  allouer2D(Bez2,ech);

  // construction de la deuxieme surface de Bezier
  printf("Construction de la surface de Bezier 2 \n");
  courbe2D(ctrl2, dim, Bez2, ech);

  // copie des gris de Bez1 dans Bez2
  printf("Copie des gris de Bez1 dans Bez2\n");
  for(i=0;i<ech;i++)
    for(j=0;j<ech;j++)
      {
	Bez2[i][j].gris = Bez1[i][j].gris;
      }

  // construction de l'image a partir des coordonnees de Bez2 et les gris de surface
  printf("Construction de l'image a partir des coordonnees de Bez2 et les gris de surface\n");
  creer_image(Im, Bez2, ech);
  
  printf("Sauvegarde de l'image : %d %d\n", Im->nb_col, Im->nb_lig);
  ecrire_image(argv[2],Im);
  free(Bez1);
  free(ctrl);
  free(ctrl2);
  sauv_bez(Im,Bez2,ech,Im->nb_col,Im->nb_lig);
  free(Im);
  free(Bez2);
  return(0);
}

